﻿unit Unit1;

interface

uses

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, System.Generics.Collections,
  REST.Json, System.JSON.Types, System.JSON.Serializers, XSuperObject;

type

  TFoo = record
    id: Integer;
    Name: string;
  end;

  TFooArray = TArray<TFoo>;

  TForm1 = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TJsonHelper = class Helper for REST.JSON.TJson
  public
    /// <summary>
    /// 对象、结构体、数组等序列化为 Json
    /// </summary>
    class function Object2Json<T>(AObject: T): string; static;
    /// <summary>
    /// Json 反序列化为对象、结构体、数组等
    /// </summary>
    class function Json2Object<T>(const AJson: string): T; static;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


{ TJsonHelper }
class function TJsonHelper.Json2Object<T>(const AJson: string): T;
begin
  with TJsonSerializer.Create do
  try
    DateTimeZoneHandling := TJsonDateTimeZoneHandling.Utc;
    // this options, serialize and deserialize work fine,
    // but I have a third party JsonString need to deserialize, special character occur unreadable code
    // TJsonSerializer.StringEscapeHandling := TJsonStringEscapeHandling.Default;
    StringEscapeHandling := TJsonStringEscapeHandling.EscapeNonAscii;
    Result := Deserialize<T>(AJson);
  finally
    Free;
  end;
end;

class function TJsonHelper.Object2Json<T>(AObject: T): string;
begin
  with TJsonSerializer.Create do
  try
    DateTimeZoneHandling := TJsonDateTimeZoneHandling.Utc;
    // TJsonSerializer.StringEscapeHandling := TJsonStringEscapeHandling.Default;
    StringEscapeHandling := TJsonStringEscapeHandling.EscapeNonAscii;
    Result := Serialize<T>(AObject);
  finally
    Free;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
// const
// JSON = '[{"id":1,"name":"\ue050\u677e\u5b50\ud83c\udfb8"},{"id":2,"name":"360\u00ba@VRM\ud83d\ude4f\ud83c\udffb"}]';
var
  Json: string;
  foo: TFoo;
  fooArray, fooArray1: TFooArray;
begin
  SetLength(fooArray, 2);
  fooArray[0].id := 1;
  fooArray[0].Name := '松子🎸';

  fooArray[1].id := 1;
  fooArray[1].Name := '360º@VRM🙏';

  // Serialize
  Json := XSuperObject.TJson.SuperObject<TFooArray>(fooArray).AsJSON();

  Memo1.Lines.Add(Json);

  // Deserialize
  fooArray1 := XSuperObject.TJson.Parse<TFooArray>(Json);
  for foo in fooArray1 do
    Memo1.Lines.Add(foo.Name); // work fine.

end;

procedure TForm1.Button2Click(Sender: TObject);
var
  Json: string;
  foo: TFoo;
  fooArray, fooArray1: TFooArray;
begin
  SetLength(fooArray, 2);
  fooArray[0].id := 1;
  fooArray[0].Name := '松子🎸';

  fooArray[1].id := 1;
  fooArray[1].Name := '360º@VRM🙏';

  // Serialize
  Json := REST.JSON.TJson.Object2Json<TFooArray>(fooArray);

  Memo1.Lines.Add(Json);

  // Deserialize
  fooArray1 := REST.JSON.TJson.Json2Object<TFooArray>(Json);
  for foo in fooArray1 do
    Memo1.Lines.Add(foo.Name); // unreadable code

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  ReportMemoryLeaksOnShutdown := True;
end;

end.
